<?php

/**
 * Migrates from the old biblio to the new biblio_cck version
 */
function biblio_cck_migrate() {
	$result = db_query('SELECT nid FROM node WHERE type="biblio"');
	$i = 0;
	while ($row = db_fetch_object($result)) {
		$node = node_load($row->nid);
		$node->type = 'biblio_base';

		unset($node->nid);
		foreach ((array)$node as $item => $value) {
			if (strpos($item, 'biblio_') === 0) {
				if (!empty($value)) {
					_biblio_field_migrate($node, $item, $value);
				}
				unset($node->$item);
			}
		}
	//	dpm($node);
		node_save($node);
		$i++;
	}
	drupal_set_message('Migrated '. $i . ' nodes');
	return '';
}


/**
 * Migrates a biblio field
 *
 * If there is a function called _biblio_field_[field name] then it will be called, else it considered a single value
 * @param Object $node
 * @param String $item
 * @param Mixed $value
 */
function _biblio_field_migrate($node, $item, $value) {
	if (function_exists('_biblio_field_' . $item)) {
		call_user_func('_biblio_field_'. $item, &$node, $value);
	} else {
		//if (!_check_field(_biblio_field_name_convert($item))) {
		//	drupal_set_message($item .' not found (' . _biblio_field_name_convert($item) .')');
		//}
		$node->{_biblio_field_name_convert($item)}[] = array('value' => $value);
	}
}


/**
 * biblio_date has to be mapped to field_year
 */
function _biblio_field_biblio_date($node, $value) {
	$date = split('/', $value);
	$timestamp = mktime(1, 1, 1, $date[0], 1, $date[1]);
	$node->field_year = array(0=> array('value' => $timestamp));
	$node->field_date_published = array(0=> array('value' => $timestamp));
}


/**
 * The author types is stored at biblio_contributor_type_data
 * This function matches the contributors to the right ccks
 */
function _biblio_field_biblio_contributors($node, $value) {
	foreach ($value as $id => $values) {
		$result = db_query('SELECT field FROM {biblio_settings} JOIN {biblio_contributor_type_data} ON title=override WHERE auth_type=%d', $id);

		if ($row = db_fetch_object($result)) {
			$field = $row->field;
		}  else {
			$field = 'field_authors';
		}

		foreach ($values as $author) {
			$node->{$field}[] = array('value' => $author['name']);
		}
	}

}


function _biblio_field_name_convert($name) {
	static $map = array( // Mapping for simple name conversion fixes
			'biblio_type_name' => 'field_publication_type',
			'biblio_isbn' => 'field_isbn_number',
			'biblio_issn' => 'field_issn_number',
			'biblio_lang' => 'field_publication_language',
			'biblio_url' => 'field_link',
			'biblio_citekey' => 'field_citation_key',
			'biblio_coins' => 'field_coins_data',
			'biblio_abst_e' => 'field_abstract',
			'biblio_abst_f' => 'field_abstract_french',
	);

	if (in_array($name, array_keys($map))) {
		return $map[$name];
	}

	return 'field_' . substr($name, strpos($name, '_')+1);
}


/**
 * Check if the field name is a valid cck field
 */
function _check_field($name) {
	$result = db_query('SELECT field_name FROM {content_node_field_instance} WHERE type_name="biblio_base" AND field_name="%s"', $name);
	if ($row = db_fetch_object($result)) {
		return TRUE;
	}
	return FALSE;
}

